# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/97_rich.ipynb.

# %% auto 0
__all__ = ['console', 'swrap', 'cprint']

# %% ../nbs/97_rich.ipynb 3
from rich.table import Table
from rich.console import Console

# %% ../nbs/97_rich.ipynb 4
console = Console()

# %% ../nbs/97_rich.ipynb 5
def swrap(style:str, text:str) -> str:
    return f'[{style}]{text}[/{style}]'

def cprint(style:str, text:str) -> None:
    console.print(swrap(style, text))
