# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/iza',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/iza',
                'lib_path': 'iza'},
  'syms': { 'iza.commands': { 'iza.commands.aggregate_notebooks': ('commands.html#aggregate_notebooks', 'iza/commands.py'),
                              'iza.commands.callback': ('commands.html#callback', 'iza/commands.py')},
            'iza.imp': { 'iza.imp.RichImp': ('imp.html#richimp', 'iza/imp.py'),
                         'iza.imp.SCPrepImp': ('imp.html#scprepimp', 'iza/imp.py'),
                         'iza.imp.ScanPyImp': ('imp.html#scanpyimp', 'iza/imp.py'),
                         'iza.imp.TorchImp': ('imp.html#torchimp', 'iza/imp.py'),
                         'iza.imp.TqdmImp': ('imp.html#tqdmimp', 'iza/imp.py'),
                         'iza.imp.is_anndata_available': ('imp.html#is_anndata_available', 'iza/imp.py'),
                         'iza.imp.is_pytorch_available': ('imp.html#is_pytorch_available', 'iza/imp.py'),
                         'iza.imp.is_rich_available': ('imp.html#is_rich_available', 'iza/imp.py')},
            'iza.nbs': { 'iza.nbs.NotebookAggregator': ('nbs.html#notebookaggregator', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.__post_init__': ('nbs.html#notebookaggregator.__post_init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.add_current_notebook_section': ( 'nbs.html#notebookaggregator.add_current_notebook_section',
                                                                                      'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.aggregate': ('nbs.html#notebookaggregator.aggregate', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.basename': ('nbs.html#notebookaggregator.basename', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.dirname': ('nbs.html#notebookaggregator.dirname', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.format_header': ('nbs.html#notebookaggregator.format_header', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.get_prefix': ('nbs.html#notebookaggregator.get_prefix', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.nb_as_href': ('nbs.html#notebookaggregator.nb_as_href', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.notebooks': ('nbs.html#notebookaggregator.notebooks', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.write_nbdev_head': ('nbs.html#notebookaggregator.write_nbdev_head', 'iza/nbs.py'),
                         'iza.nbs.NotebookAggregator.write_nbdev_tail': ('nbs.html#notebookaggregator.write_nbdev_tail', 'iza/nbs.py'),
                         'iza.nbs.NotebookFinder': ('nbs.html#notebookfinder', 'iza/nbs.py'),
                         'iza.nbs.NotebookFinder.__init__': ('nbs.html#notebookfinder.__init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookFinder.find_module': ('nbs.html#notebookfinder.find_module', 'iza/nbs.py'),
                         'iza.nbs.NotebookLoader': ('nbs.html#notebookloader', 'iza/nbs.py'),
                         'iza.nbs.NotebookLoader.__init__': ('nbs.html#notebookloader.__init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookLoader.load_module': ('nbs.html#notebookloader.load_module', 'iza/nbs.py'),
                         'iza.nbs.NotebookLogger': ('nbs.html#notebooklogger', 'iza/nbs.py'),
                         'iza.nbs.NotebookLogger.__del__': ('nbs.html#notebooklogger.__del__', 'iza/nbs.py'),
                         'iza.nbs.NotebookLogger.__post_init__': ('nbs.html#notebooklogger.__post_init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer': ('nbs.html#notebookprefixer', 'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer.__post_init__': ('nbs.html#notebookprefixer.__post_init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer.filename_without_prefix': ( 'nbs.html#notebookprefixer.filename_without_prefix',
                                                                               'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer.int_to_prefix_int': ('nbs.html#notebookprefixer.int_to_prefix_int', 'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer.next_nb_prefix': ('nbs.html#notebookprefixer.next_nb_prefix', 'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer.prefix_from_filename': ('nbs.html#notebookprefixer.prefix_from_filename', 'iza/nbs.py'),
                         'iza.nbs.NotebookPrefixer.prefix_int_to_int': ('nbs.html#notebookprefixer.prefix_int_to_int', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer': ('nbs.html#notebookviewer', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer.__init__': ('nbs.html#notebookviewer.__init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer.__post_init__': ('nbs.html#notebookviewer.__post_init__', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer.get_cell': ('nbs.html#notebookviewer.get_cell', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer.prepare_cell': ('nbs.html#notebookviewer.prepare_cell', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer.show_cell': ('nbs.html#notebookviewer.show_cell', 'iza/nbs.py'),
                         'iza.nbs.NotebookViewer.show_notebook': ('nbs.html#notebookviewer.show_notebook', 'iza/nbs.py'),
                         'iza.nbs.find_notebook': ('nbs.html#find_notebook', 'iza/nbs.py')},
            'iza.plots': { 'iza.plots.convert_cmap_to_categorical': ('plots.html#convert_cmap_to_categorical', 'iza/plots.py'),
                           'iza.plots.hide_axes': ('plots.html#hide_axes', 'iza/plots.py'),
                           'iza.plots.make_categorical_legend': ('plots.html#make_categorical_legend', 'iza/plots.py'),
                           'iza.plots.mpl_cat_val': ('plots.html#mpl_cat_val', 'iza/plots.py'),
                           'iza.plots.mpl_cur_cmap': ('plots.html#mpl_cur_cmap', 'iza/plots.py'),
                           'iza.plots.scale_figsize_for_grid': ('plots.html#scale_figsize_for_grid', 'iza/plots.py'),
                           'iza.plots.set_rc_params': ('plots.html#set_rc_params', 'iza/plots.py'),
                           'iza.plots.setup_grid_figure': ('plots.html#setup_grid_figure', 'iza/plots.py'),
                           'iza.plots.triple_scatter': ('plots.html#triple_scatter', 'iza/plots.py')},
            'iza.rich': {'iza.rich.cprint': ('rich.html#cprint', 'iza/rich.py'), 'iza.rich.swrap': ('rich.html#swrap', 'iza/rich.py')},
            'iza.static': {},
            'iza.typer': {},
            'iza.types': { 'iza.types.DirectoryTreeStrings': ('types.html#directorytreestrings', 'iza/types.py'),
                           'iza.types.TrainTestSplit': ('types.html#traintestsplit', 'iza/types.py'),
                           'iza.types.TrainValidSplit': ('types.html#trainvalidsplit', 'iza/types.py'),
                           'iza.types.TrainValidTestSplit': ('types.html#trainvalidtestsplit', 'iza/types.py')},
            'iza.utils': { 'iza.utils.AdataExtractor': ('utils.html#adataextractor', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.axis_str': ('utils.html#adataextractor.axis_str', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.df_cnt': ('utils.html#adataextractor.df_cnt', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.df_emb': ('utils.html#adataextractor.df_emb', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.emb_cols': ('utils.html#adataextractor.emb_cols', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.get_df_cnt': ('utils.html#adataextractor.get_df_cnt', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.get_df_emb': ('utils.html#adataextractor.get_df_emb', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.get_emb': ('utils.html#adataextractor.get_emb', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.get_layer': ('utils.html#adataextractor.get_layer', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.has_emb': ('utils.html#adataextractor.has_emb', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.has_hvg': ('utils.html#adataextractor.has_hvg', 'iza/utils.py'),
                           'iza.utils.AdataExtractor.sdata': ('utils.html#adataextractor.sdata', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader': ('utils.html#archivedownloader', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.__post_init__': ('utils.html#archivedownloader.__post_init__', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.calc_n_to_extract': ( 'utils.html#archivedownloader.calc_n_to_extract',
                                                                              'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.download_missing_files': ( 'utils.html#archivedownloader.download_missing_files',
                                                                                   'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.execute': ('utils.html#archivedownloader.execute', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.extract_files': ('utils.html#archivedownloader.extract_files', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.get_progress': ('utils.html#archivedownloader.get_progress', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.path': ('utils.html#archivedownloader.path', 'iza/utils.py'),
                           'iza.utils.ArchiveDownloader.urls': ('utils.html#archivedownloader.urls', 'iza/utils.py'),
                           'iza.utils.Directory': ('utils.html#directory', 'iza/utils.py'),
                           'iza.utils.DirectoryTree': ('utils.html#directorytree', 'iza/utils.py'),
                           'iza.utils.DirectoryTree.__post_init__': ('utils.html#directorytree.__post_init__', 'iza/utils.py'),
                           'iza.utils.DirectoryTree.__repr__': ('utils.html#directorytree.__repr__', 'iza/utils.py'),
                           'iza.utils.DirectoryTree.get_tree_lines': ('utils.html#directorytree.get_tree_lines', 'iza/utils.py'),
                           'iza.utils.DirectoryTree.make_tree_str': ('utils.html#directorytree.make_tree_str', 'iza/utils.py'),
                           'iza.utils.DirectoryTree.print': ('utils.html#directorytree.print', 'iza/utils.py'),
                           'iza.utils.DirectoryTree.tree_generator': ('utils.html#directorytree.tree_generator', 'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor': ('utils.html#recursivedecompressor', 'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor.__post_init__': ( 'utils.html#recursivedecompressor.__post_init__',
                                                                              'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor.decompress': ('utils.html#recursivedecompressor.decompress', 'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor.decrease_total': ( 'utils.html#recursivedecompressor.decrease_total',
                                                                               'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor.execute': ('utils.html#recursivedecompressor.execute', 'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor.recurse': ('utils.html#recursivedecompressor.recurse', 'iza/utils.py'),
                           'iza.utils.RecursiveDecompressor.update_total': ( 'utils.html#recursivedecompressor.update_total',
                                                                             'iza/utils.py'),
                           'iza.utils.RichDirectory': ('utils.html#richdirectory', 'iza/utils.py'),
                           'iza.utils.RichDirectory.__post_init__': ('utils.html#richdirectory.__post_init__', 'iza/utils.py'),
                           'iza.utils.RichDirectory.print_rich': ('utils.html#richdirectory.print_rich', 'iza/utils.py'),
                           'iza.utils.Slice': ('utils.html#slice', 'iza/utils.py'),
                           'iza.utils.Slice.__post_init__': ('utils.html#slice.__post_init__', 'iza/utils.py'),
                           'iza.utils.Slice.astype': ('utils.html#slice.astype', 'iza/utils.py'),
                           'iza.utils.Slice.start': ('utils.html#slice.start', 'iza/utils.py'),
                           'iza.utils.Slice.step': ('utils.html#slice.step', 'iza/utils.py'),
                           'iza.utils.Slice.stop': ('utils.html#slice.stop', 'iza/utils.py'),
                           'iza.utils.Slice.todict': ('utils.html#slice.todict', 'iza/utils.py'),
                           'iza.utils.Slice.tolist': ('utils.html#slice.tolist', 'iza/utils.py'),
                           'iza.utils.Slice.toslice': ('utils.html#slice.toslice', 'iza/utils.py'),
                           'iza.utils.Slice.totuple': ('utils.html#slice.totuple', 'iza/utils.py'),
                           'iza.utils.allinstance': ('utils.html#allinstance', 'iza/utils.py'),
                           'iza.utils.allsametype': ('utils.html#allsametype', 'iza/utils.py'),
                           'iza.utils.arein': ('utils.html#arein', 'iza/utils.py'),
                           'iza.utils.base_entry_fn': ('utils.html#base_entry_fn', 'iza/utils.py'),
                           'iza.utils.base_init_tree': ('utils.html#base_init_tree', 'iza/utils.py'),
                           'iza.utils.check_ext': ('utils.html#check_ext', 'iza/utils.py'),
                           'iza.utils.collapse_user': ('utils.html#collapse_user', 'iza/utils.py'),
                           'iza.utils.config_exp_logger': ('utils.html#config_exp_logger', 'iza/utils.py'),
                           'iza.utils.copy_to_clipboard': ('utils.html#copy_to_clipboard', 'iza/utils.py'),
                           'iza.utils.decompress_directory_of_gunzipped_files': ( 'utils.html#decompress_directory_of_gunzipped_files',
                                                                                  'iza/utils.py'),
                           'iza.utils.decompress_gunzip': ('utils.html#decompress_gunzip', 'iza/utils.py'),
                           'iza.utils.decompress_tarball': ('utils.html#decompress_tarball', 'iza/utils.py'),
                           'iza.utils.decompress_tarball_of_gunzipped_files': ( 'utils.html#decompress_tarball_of_gunzipped_files',
                                                                                'iza/utils.py'),
                           'iza.utils.dir_dirs': ('utils.html#dir_dirs', 'iza/utils.py'),
                           'iza.utils.download_and_decompress_tarball_of_gunzipped_files': ( 'utils.html#download_and_decompress_tarball_of_gunzipped_files',
                                                                                             'iza/utils.py'),
                           'iza.utils.drop_ext': ('utils.html#drop_ext', 'iza/utils.py'),
                           'iza.utils.entry_fn': ('utils.html#entry_fn', 'iza/utils.py'),
                           'iza.utils.exp_log_filename': ('utils.html#exp_log_filename', 'iza/utils.py'),
                           'iza.utils.exp_param_filename': ('utils.html#exp_param_filename', 'iza/utils.py'),
                           'iza.utils.filter_for_gz_files': ('utils.html#filter_for_gz_files', 'iza/utils.py'),
                           'iza.utils.filter_kwargs_for_class': ('utils.html#filter_kwargs_for_class', 'iza/utils.py'),
                           'iza.utils.filter_kwargs_for_func': ('utils.html#filter_kwargs_for_func', 'iza/utils.py'),
                           'iza.utils.find_exps': ('utils.html#find_exps', 'iza/utils.py'),
                           'iza.utils.gen_exp_name': ('utils.html#gen_exp_name', 'iza/utils.py'),
                           'iza.utils.get_func_params': ('utils.html#get_func_params', 'iza/utils.py'),
                           'iza.utils.get_gz_files_in_dir': ('utils.html#get_gz_files_in_dir', 'iza/utils.py'),
                           'iza.utils.get_user': ('utils.html#get_user', 'iza/utils.py'),
                           'iza.utils.init_tree': ('utils.html#init_tree', 'iza/utils.py'),
                           'iza.utils.is_config_subset': ('utils.html#is_config_subset', 'iza/utils.py'),
                           'iza.utils.is_cpu': ('utils.html#is_cpu', 'iza/utils.py'),
                           'iza.utils.is_device': ('utils.html#is_device', 'iza/utils.py'),
                           'iza.utils.is_gz': ('utils.html#is_gz', 'iza/utils.py'),
                           'iza.utils.is_matrix': ('utils.html#is_matrix', 'iza/utils.py'),
                           'iza.utils.is_mps': ('utils.html#is_mps', 'iza/utils.py'),
                           'iza.utils.is_np': ('utils.html#is_np', 'iza/utils.py'),
                           'iza.utils.is_path': ('utils.html#is_path', 'iza/utils.py'),
                           'iza.utils.is_pathlib': ('utils.html#is_pathlib', 'iza/utils.py'),
                           'iza.utils.is_pathlike': ('utils.html#is_pathlike', 'iza/utils.py'),
                           'iza.utils.is_series': ('utils.html#is_series', 'iza/utils.py'),
                           'iza.utils.is_series_like': ('utils.html#is_series_like', 'iza/utils.py'),
                           'iza.utils.is_tar': ('utils.html#is_tar', 'iza/utils.py'),
                           'iza.utils.is_tarball': ('utils.html#is_tarball', 'iza/utils.py'),
                           'iza.utils.is_targz': ('utils.html#is_targz', 'iza/utils.py'),
                           'iza.utils.is_tensor': ('utils.html#is_tensor', 'iza/utils.py'),
                           'iza.utils.is_torch': ('utils.html#is_torch', 'iza/utils.py'),
                           'iza.utils.isin': ('utils.html#isin', 'iza/utils.py'),
                           'iza.utils.isiter': ('utils.html#isiter', 'iza/utils.py'),
                           'iza.utils.list_exps': ('utils.html#list_exps', 'iza/utils.py'),
                           'iza.utils.load_exp_params': ('utils.html#load_exp_params', 'iza/utils.py'),
                           'iza.utils.make_missing_dirs': ('utils.html#make_missing_dirs', 'iza/utils.py'),
                           'iza.utils.make_temp_file': ('utils.html#make_temp_file', 'iza/utils.py'),
                           'iza.utils.path_to_str': ('utils.html#path_to_str', 'iza/utils.py'),
                           'iza.utils.rich_entry_fn': ('utils.html#rich_entry_fn', 'iza/utils.py'),
                           'iza.utils.rich_init_tree': ('utils.html#rich_init_tree', 'iza/utils.py'),
                           'iza.utils.save_exp_params': ('utils.html#save_exp_params', 'iza/utils.py'),
                           'iza.utils.setup_exp': ('utils.html#setup_exp', 'iza/utils.py'),
                           'iza.utils.sort_directory_first': ('utils.html#sort_directory_first', 'iza/utils.py'),
                           'iza.utils.sort_file_first': ('utils.html#sort_file_first', 'iza/utils.py'),
                           'iza.utils.str_just_alpha': ('utils.html#str_just_alpha', 'iza/utils.py'),
                           'iza.utils.str_just_numeric': ('utils.html#str_just_numeric', 'iza/utils.py'),
                           'iza.utils.str_part_end': ('utils.html#str_part_end', 'iza/utils.py'),
                           'iza.utils.str_part_mid': ('utils.html#str_part_mid', 'iza/utils.py'),
                           'iza.utils.str_part_pre': ('utils.html#str_part_pre', 'iza/utils.py'),
                           'iza.utils.str_split_get': ('utils.html#str_split_get', 'iza/utils.py'),
                           'iza.utils.str_to_path': ('utils.html#str_to_path', 'iza/utils.py'),
                           'iza.utils.stream_file': ('utils.html#stream_file', 'iza/utils.py'),
                           'iza.utils.strip_punc': ('utils.html#strip_punc', 'iza/utils.py'),
                           'iza.utils.to_abs_expanded': ('utils.html#to_abs_expanded', 'iza/utils.py'),
                           'iza.utils.to_ndarray': ('utils.html#to_ndarray', 'iza/utils.py'),
                           'iza.utils.undo_gz': ('utils.html#undo_gz', 'iza/utils.py'),
                           'iza.utils.undo_npmatrix': ('utils.html#undo_npmatrix', 'iza/utils.py'),
                           'iza.utils.undo_sparse': ('utils.html#undo_sparse', 'iza/utils.py'),
                           'iza.utils.urljoin': ('utils.html#urljoin', 'iza/utils.py'),
                           'iza.utils.walk_dir_tree': ('utils.html#walk_dir_tree', 'iza/utils.py'),
                           'iza.utils.wrangle_kwargs_for_class': ('utils.html#wrangle_kwargs_for_class', 'iza/utils.py'),
                           'iza.utils.wrangle_kwargs_for_func': ('utils.html#wrangle_kwargs_for_func', 'iza/utils.py')}}}
